@@ -31,16 +31,16 @@ use super::{
     cursor::Cursor,
     engines::{SearchEngine, SearchState},
     history_list::{HistoryList, ListState, PREFIX_LENGTH},
-    sort,
 };
 
+use crate::command::client::theme::{Meaning, Theme};
 use crate::{command::client::search::engines, VERSION};
 
 use ratatui::{
     backend::CrosstermBackend,
     layout::{Alignment, Constraint, Direction, Layout},
     prelude::*,
-    style::{Color, Modifier, Style},
+    style::{Modifier, Style},
     text::{Line, Span, Text},
     widgets::{block::Title, Block, BorderType, Borders, Padding, Paragraph, Tabs},
     Frame, Terminal, TerminalOptions, Viewport,
@@ -96,7 +96,10 @@ impl State {
         self.results_len = results.len();
 
         if smart_sort {
-            Ok(sort::sort(self.search.input.as_str(), results))
+            Ok(atuin_history::sort::sort(
+                self.search.input.as_str(),
+                results,
+            ))
         } else {
             Ok(results)
         }
@@ -202,9 +205,15 @@ impl State {
 
         let ctrl = input.modifiers.contains(KeyModifiers::CONTROL);
         let esc_allow_exit = !(self.tab_index == 0 && self.keymap_mode == KeymapMode::VimInsert);
+        let cursor_at_end_of_line =
+            self.search.input.position() == UnicodeWidthStr::width(self.search.input.as_str());
+        let cursor_at_start_of_line = self.search.input.position() == 0;
 
         // support ctrl-a prefix, like screen or tmux
-        if ctrl && input.code == KeyCode::Char('a') {
+        if !self.prefix
+            && ctrl
+            && input.code == KeyCode::Char(settings.keys.prefix.chars().next().unwrap_or('a'))
+        {
             self.prefix = true;
             return InputAction::Continue;
         }
@@ -215,12 +224,14 @@ impl State {
             KeyCode::Esc if esc_allow_exit => Some(Self::handle_key_exit(settings)),
             KeyCode::Char('[') if ctrl && esc_allow_exit => Some(Self::handle_key_exit(settings)),
             KeyCode::Tab => Some(InputAction::Accept(self.results_state.selected())),
+            KeyCode::Right if cursor_at_end_of_line => {
+                Some(InputAction::Accept(self.results_state.selected()))
+            }
+            KeyCode::Left if cursor_at_start_of_line => Some(Self::handle_key_exit(settings)),
             KeyCode::Char('o') if ctrl => {
                 self.tab_index = (self.tab_index + 1) % TAB_TITLES.len();
-
                 Some(InputAction::Continue)
             }
-
             _ => None,
         };
 
@@ -299,6 +310,8 @@ impl State {
                 }
                 KeyCode::Char('a') => {
                     self.search.input.start();
+                    //  This prevents pressing ctrl-a twice while still in prefix mode
+                    self.prefix = false;
                     return InputAction::Continue;
                 }
                 _ => {}
@@ -451,29 +464,7 @@ impl State {
                 }
             }
             KeyCode::Char('u') if ctrl => self.search.input.clear(),
-            KeyCode::Char('r') if ctrl => {
-                let filter_modes = if settings.workspaces && self.search.context.git_root.is_some()
-                {
-                    vec![
-                        FilterMode::Global,
-                        FilterMode::Host,
-                        FilterMode::Session,
-                        FilterMode::Directory,
-                        FilterMode::Workspace,
-                    ]
-                } else {
-                    vec![
-                        FilterMode::Global,
-                        FilterMode::Host,
-                        FilterMode::Session,
-                        FilterMode::Directory,
-                    ]
-                };
-
-                let i = self.search.filter_mode as usize;
-                let i = (i + 1) % filter_modes.len();
-                self.search.filter_mode = filter_modes[i];
-            }
+            KeyCode::Char('r') if ctrl => self.search.rotate_filter_mode(settings, 1),
             KeyCode::Char('s') if ctrl => {
                 self.switched_search_mode = true;
                 self.search_mode = self.search_mode.next(settings);
@@ -526,7 +517,8 @@ impl State {
 
     fn scroll_up(&mut self, scroll_len: usize) {
         let i = self.results_state.selected() + scroll_len;
-        self.results_state.select(i.min(self.results_len - 1));
+        self.results_state
+            .select(i.min(self.results_len.saturating_sub(1)));
     }
 
     #[allow(clippy::cast_possible_truncation)]
@@ -575,6 +567,8 @@ impl State {
                         .sum(),
                 )
             }) + border_size * 2
+        } else if settings.show_preview && settings.preview.strategy == PreviewStrategy::Fixed {
+            settings.max_preview_height + border_size * 2
         } else if compact || tab_index == 1 {
             0
         } else {
@@ -591,6 +585,7 @@ impl State {
         results: &[History],
         stats: Option<HistoryStats>,
         settings: &Settings,
+        theme: &Theme,
     ) {
         let compact = match settings.style {
             atuin_client::settings::Style::Auto => f.size().height < 14,
@@ -610,12 +605,17 @@ impl State {
             preview_width,
         );
         let show_help = settings.show_help && (!compact || f.size().height > 1);
-        let show_tabs = settings.show_tabs;
+        // This is an OR, as it seems more likely for someone to wish to override
+        // tabs unexpectedly being missed, than unexpectedly present.
+        let hide_extra = settings.auto_hide_height != 0
+            && compact
+            && f.size().height <= settings.auto_hide_height;
+        let show_tabs = settings.show_tabs && !hide_extra;
         let chunks = Layout::default()
             .direction(Direction::Vertical)
             .margin(0)
             .horizontal_margin(1)
-            .constraints(
+            .constraints::<&[Constraint]>(
                 if invert {
                     [
                         Constraint::Length(1 + border_size),               // input
@@ -624,6 +624,14 @@ impl State {
                         Constraint::Length(if show_tabs { 1 } else { 0 }), // tabs
                         Constraint::Length(if show_help { 1 } else { 0 }), // header (sic)
                     ]
+                } else if hide_extra {
+                    [
+                        Constraint::Length(if show_help { 1 } else { 0 }), // header
+                        Constraint::Length(0),                             // tabs
+                        Constraint::Min(1),                                // results list
+                        Constraint::Length(0),
+                        Constraint::Length(0),
+                    ]
                 } else {
                     [
                         Constraint::Length(if show_help { 1 } else { 0 }), // header
@@ -648,13 +656,15 @@ impl State {
         // also allocate less 🙈
         let titles: Vec<_> = TAB_TITLES.iter().copied().map(Line::from).collect();
 
-        let tabs = Tabs::new(titles)
-            .block(Block::default().borders(Borders::NONE))
-            .select(self.tab_index)
-            .style(Style::default())
-            .highlight_style(Style::default().bold().white().on_black());
+        if show_tabs {
+            let tabs = Tabs::new(titles)
+                .block(Block::default().borders(Borders::NONE))
+                .select(self.tab_index)
+                .style(Style::default())
+                .highlight_style(Style::default().bold().white().on_black());
 
-        f.render_widget(tabs, tabs_chunk);
+            f.render_widget(tabs, tabs_chunk);
+        }
 
         let style = StyleState {
             compact,
@@ -664,7 +674,7 @@ impl State {
 
         let header_chunks = Layout::default()
             .direction(Direction::Horizontal)
-            .constraints(
+            .constraints::<&[Constraint]>(
                 [
                     Constraint::Ratio(1, 5),
                     Constraint::Ratio(3, 5),
@@ -674,19 +684,36 @@ impl State {
             )
             .split(header_chunk);
 
-        let title = self.build_title();
+        let title = self.build_title(theme);
         f.render_widget(title, header_chunks[0]);
 
-        let help = self.build_help(settings);
+        let help = self.build_help(settings, theme);
         f.render_widget(help, header_chunks[1]);
 
-        let stats_tab = self.build_stats();
+        let stats_tab = self.build_stats(theme);
         f.render_widget(stats_tab, header_chunks[2]);
 
+        let indicator: String = if !hide_extra {
+            " > ".to_string()
+        } else if self.switched_search_mode {
+            format!("S{}>", self.search_mode.as_str().chars().next().unwrap())
+        } else {
+            format!(
+                "{}> ",
+                self.search.filter_mode.as_str().chars().next().unwrap()
+            )
+        };
+
         match self.tab_index {
             0 => {
-                let results_list =
-                    Self::build_results_list(style, results, self.keymap_mode, &self.now);
+                let results_list = Self::build_results_list(
+                    style,
+                    results,
+                    self.keymap_mode,
+                    &self.now,
+                    indicator.as_str(),
+                    theme,
+                );
                 f.render_stateful_widget(results_list, results_list_chunk, &mut self.results_state);
             }
 
@@ -709,6 +736,7 @@ impl State {
                         results_list_chunk,
                         &results[self.results_state.selected()],
                         &stats.expect("Drawing inspector, but no stats"),
+                        theme,
                     );
                 }
 
@@ -725,45 +753,54 @@ impl State {
             }
         }
 
-        let input = self.build_input(style);
-        f.render_widget(input, input_chunk);
+        if !hide_extra {
+            let input = self.build_input(style);
+            f.render_widget(input, input_chunk);
 
-        let preview_width = if compact {
-            preview_width
-        } else {
-            preview_width - 2
-        };
-        let preview =
-            self.build_preview(results, compact, preview_width, preview_chunk.width.into());
-        f.render_widget(preview, preview_chunk);
+            let preview_width = if compact {
+                preview_width
+            } else {
+                preview_width - 2
+            };
+            let preview = self.build_preview(
+                results,
+                compact,
+                preview_width,
+                preview_chunk.width.into(),
+                theme,
+            );
+            f.render_widget(preview, preview_chunk);
 
-        let extra_width = UnicodeWidthStr::width(self.search.input.substring());
+            let extra_width = UnicodeWidthStr::width(self.search.input.substring());
 
-        let cursor_offset = if compact { 0 } else { 1 };
-        f.set_cursor(
-            // Put cursor past the end of the input text
-            input_chunk.x + extra_width as u16 + PREFIX_LENGTH + 1 + cursor_offset,
-            input_chunk.y + cursor_offset,
-        );
+            let cursor_offset = if compact { 0 } else { 1 };
+            f.set_cursor(
+                // Put cursor past the end of the input text
+                input_chunk.x + extra_width as u16 + PREFIX_LENGTH + 1 + cursor_offset,
+                input_chunk.y + cursor_offset,
+            );
+        }
     }
 
-    fn build_title(&mut self) -> Paragraph {
+    fn build_title(&self, theme: &Theme) -> Paragraph {
         let title = if self.update_needed.is_some() {
+            let error_style: Style = theme.get_error().into();
             Paragraph::new(Text::from(Span::styled(
                 format!("Atuin v{VERSION} - UPGRADE"),
-                Style::default().add_modifier(Modifier::BOLD).fg(Color::Red),
+                error_style.add_modifier(Modifier::BOLD),
             )))
         } else {
+            let style: Style = theme.as_style(Meaning::Base).into();
             Paragraph::new(Text::from(Span::styled(
                 format!("Atuin v{VERSION}"),
-                Style::default().add_modifier(Modifier::BOLD),
+                style.add_modifier(Modifier::BOLD),
             )))
         };
         title.alignment(Alignment::Left)
     }
 
     #[allow(clippy::unused_self)]
-    fn build_help(&self, settings: &Settings) -> Paragraph {
+    fn build_help(&self, settings: &Settings, theme: &Theme) -> Paragraph {
         match self.tab_index {
             // search
             0 => Paragraph::new(Text::from(Line::from(vec![
@@ -797,16 +834,16 @@ impl State {
 
             _ => unreachable!("invalid tab index"),
         }
-        .style(Style::default().fg(Color::DarkGray))
+        .style(theme.as_style(Meaning::Annotation))
         .alignment(Alignment::Center)
     }
 
-    fn build_stats(&mut self) -> Paragraph {
+    fn build_stats(&self, theme: &Theme) -> Paragraph {
         let stats = Paragraph::new(Text::from(Span::raw(format!(
             "history count: {}",
             self.history_count,
         ))))
-        .style(Style::default().fg(Color::DarkGray))
+        .style(theme.as_style(Meaning::Annotation))
         .alignment(Alignment::Right);
         stats
     }
@@ -816,12 +853,16 @@ impl State {
         results: &'a [History],
         keymap_mode: KeymapMode,
         now: &'a dyn Fn() -> OffsetDateTime,
+        indicator: &'a str,
+        theme: &'a Theme,
     ) -> HistoryList<'a> {
         let results_list = HistoryList::new(
             results,
             style.invert,
             keymap_mode == KeymapMode::VimNormal,
             now,
+            indicator,
+            theme,
         );
 
         if style.compact {
@@ -842,7 +883,7 @@ impl State {
         }
     }
 
-    fn build_input(&mut self, style: StyleState) -> Paragraph {
+    fn build_input(&self, style: StyleState) -> Paragraph {
         /// Max width of the UI box showing current mode
         const MAX_WIDTH: usize = 14;
         let (pref, mode) = if self.switched_search_mode {
@@ -874,11 +915,12 @@ impl State {
     }
 
     fn build_preview(
-        &mut self,
+        &self,
         results: &[History],
         compact: bool,
         preview_width: u16,
         chunk_width: usize,
+        theme: &Theme,
     ) -> Paragraph {
         let selected = self.results_state.selected();
         let command = if results.is_empty() {
@@ -898,7 +940,7 @@ impl State {
                 .join("\n")
         };
         let preview = if compact {
-            Paragraph::new(command).style(Style::default().fg(Color::DarkGray))
+            Paragraph::new(command).style(theme.as_style(Meaning::Annotation))
         } else {
             Paragraph::new(command).block(
                 Block::default()
@@ -986,6 +1028,7 @@ pub async fn history(
     settings: &Settings,
     mut db: impl Database,
     history_store: &HistoryStore,
+    theme: &Theme,
 ) -> Result<String> {
     let stdout = Stdout::new(settings.inline_height > 0)?;
     let backend = CrosstermBackend::new(stdout);
@@ -1027,15 +1070,12 @@ pub async fn history(
         tab_index: 0,
         search: SearchState {
             input,
-            filter_mode: if settings.workspaces && context.git_root.is_some() {
-                FilterMode::Workspace
-            } else if settings.shell_up_key_binding {
-                settings
-                    .filter_mode_shell_up_key_binding
-                    .unwrap_or(settings.filter_mode)
-            } else {
-                settings.filter_mode
-            },
+            filter_mode: settings
+                .filter_mode_shell_up_key_binding
+                .filter(|_| settings.shell_up_key_binding)
+                .or_else(|| Some(settings.default_filter_mode()))
+                .filter(|&x| x != FilterMode::Workspace || context.git_root.is_some())
+                .unwrap_or(FilterMode::Global),
             context,
         },
         engine: engines::engine(search_mode),
@@ -1062,7 +1102,7 @@ pub async fn history(
     let mut stats: Option<HistoryStats> = None;
     let accept;
     let result = 'render: loop {
-        terminal.draw(|f| app.draw(f, &results, stats.clone(), settings))?;
+        terminal.draw(|f| app.draw(f, &results, stats.clone(), settings, theme))?;
 
         let initial_input = app.search.input.as_str().to_owned();
         let initial_filter_mode = app.search.filter_mode;
@@ -1096,7 +1136,7 @@ pub async fn history(
                             },
                             InputAction::Redraw => {
                                 terminal.clear()?;
-                                terminal.draw(|f| app.draw(f, &results, stats.clone(), settings))?;
+                                terminal.draw(|f| app.draw(f, &results, stats.clone(), settings, theme))?;
                             },
                             r => {
                                 accept = app.accept;
@@ -1174,7 +1214,10 @@ pub async fn history(
     any(target_os = "windows", target_os = "macos", target_os = "linux")
 ))]
 fn set_clipboard(s: String) {
-    cli_clipboard::set_contents(s).unwrap();
+    let mut ctx = arboard::Clipboard::new().unwrap();
+    ctx.set_text(s).unwrap();
+    // Use the clipboard context to make sure it is saved
+    ctx.get_text().unwrap();
 }
 
 #[cfg(not(all(
@@ -1185,8 +1228,15 @@ fn set_clipboard(_s: String) {}
 
 #[cfg(test)]
 mod tests {
+    use atuin_client::database::Context;
     use atuin_client::history::History;
-    use atuin_client::settings::{Preview, PreviewStrategy, Settings};
+    use atuin_client::settings::{
+        FilterMode, KeymapMode, Preview, PreviewStrategy, SearchMode, Settings,
+    };
+    use time::OffsetDateTime;
+
+    use crate::command::client::search::engines::{self, SearchState};
+    use crate::command::client::search::history_list::ListState;
 
     use super::State;
 
@@ -1218,6 +1268,15 @@ mod tests {
             ..Settings::utc()
         };
 
+        let settings_preview_fixed = Settings {
+            preview: Preview {
+                strategy: PreviewStrategy::Fixed,
+            },
+            show_preview: true,
+            max_preview_height: 15,
+            ..Settings::utc()
+        };
+
         let cmd_60: History = History::capture()
             .timestamp(time::OffsetDateTime::now_utc())
             .command("for i in $(seq -w 10); do echo \"item number $i - abcd\"; done")
@@ -1245,8 +1304,8 @@ mod tests {
         let no_preview = State::calc_preview_height(
             &settings_preview_auto,
             &results,
-            0 as usize,
-            0 as usize,
+            0_usize,
+            0_usize,
             false,
             1,
             80,
@@ -1255,8 +1314,8 @@ mod tests {
         let preview_h2 = State::calc_preview_height(
             &settings_preview_auto,
             &results,
-            1 as usize,
-            0 as usize,
+            1_usize,
+            0_usize,
             false,
             1,
             80,
@@ -1265,8 +1324,8 @@ mod tests {
         let preview_h3 = State::calc_preview_height(
             &settings_preview_auto,
             &results,
-            2 as usize,
-            0 as usize,
+            2_usize,
+            0_usize,
             false,
             1,
             80,
@@ -1275,8 +1334,8 @@ mod tests {
         let preview_one_line = State::calc_preview_height(
             &settings_preview_auto,
             &results,
-            0 as usize,
-            0 as usize,
+            0_usize,
+            0_usize,
             false,
             1,
             66,
@@ -1285,8 +1344,8 @@ mod tests {
         let preview_limit_at_2 = State::calc_preview_height(
             &settings_preview_auto_h2,
             &results,
-            2 as usize,
-            0 as usize,
+            2_usize,
+            0_usize,
             false,
             1,
             80,
@@ -1295,8 +1354,8 @@ mod tests {
         let preview_static_h3 = State::calc_preview_height(
             &settings_preview_h4,
             &results,
-            1 as usize,
-            0 as usize,
+            1_usize,
+            0_usize,
             false,
             1,
             80,
@@ -1305,20 +1364,66 @@ mod tests {
         let preview_static_limit_at_4 = State::calc_preview_height(
             &settings_preview_h4,
             &results,
-            1 as usize,
-            0 as usize,
+            1_usize,
+            0_usize,
+            false,
+            1,
+            20,
+        );
+        // the longest command requires 10 lines, but we have a max preview height of 15 and a fixed preview strategy
+        let settings_preview_fixed = State::calc_preview_height(
+            &settings_preview_fixed,
+            &results,
+            1_usize,
+            0_usize,
             false,
             1,
             20,
         );
 
         assert_eq!(no_preview, 1);
-        // 1*2 is the space for the border
-        assert_eq!(preview_h2, 2 + 1 * 2);
-        assert_eq!(preview_h3, 3 + 1 * 2);
-        assert_eq!(preview_one_line, 1 + 1 * 2);
-        assert_eq!(preview_limit_at_2, 2 + 1 * 2);
-        assert_eq!(preview_static_h3, 3 + 1 * 2);
-        assert_eq!(preview_static_limit_at_4, 4 + 1 * 2);
+        // 1 * 2 is the space for the border
+        let border_space = 2;
+        assert_eq!(preview_h2, 2 + border_space);
+        assert_eq!(preview_h3, 3 + border_space);
+        assert_eq!(preview_one_line, 1 + border_space);
+        assert_eq!(preview_limit_at_2, 2 + border_space);
+        assert_eq!(preview_static_h3, 3 + border_space);
+        assert_eq!(preview_static_limit_at_4, 4 + border_space);
+        assert_eq!(settings_preview_fixed, 15 + border_space);
+    }
+
+    // Test when there's no results, scrolling up or down doesn't underflow
+    #[test]
+    fn state_scroll_up_underflow() {
+        let mut state = State {
+            history_count: 0,
+            update_needed: None,
+            results_state: ListState::default(),
+            switched_search_mode: false,
+            search_mode: SearchMode::Fuzzy,
+            results_len: 0,
+            accept: false,
+            keymap_mode: KeymapMode::Auto,
+            prefix: false,
+            current_cursor: None,
+            tab_index: 0,
+            search: SearchState {
+                input: String::new().into(),
+                filter_mode: FilterMode::Directory,
+                context: Context {
+                    session: String::new(),
+                    cwd: String::new(),
+                    hostname: String::new(),
+                    host_id: String::new(),
+                    git_root: None,
+                },
+            },
+            engine: engines::engine(SearchMode::Fuzzy),
+            now: Box::new(OffsetDateTime::now_utc),
+        };
+
+        state.scroll_up(1);
+        state.scroll_down(1);
     }
 }
